

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

A #GdkKeymapKey is a hardware key that can be mapped to a keyval.
-}

module GI.Gdk.Structs.KeymapKey
    ( 

-- * Exported types
    KeymapKey(..)                           ,
    newZeroKeymapKey                        ,
    noKeymapKey                             ,


 -- * Properties
-- ** Group
    keymapKeyReadGroup                      ,


-- ** Keycode
    keymapKeyReadKeycode                    ,


-- ** Level
    keymapKeyReadLevel                      ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gdk.Types
import GI.Gdk.Callbacks

newtype KeymapKey = KeymapKey (ForeignPtr KeymapKey)
-- | Construct a `KeymapKey` struct initialized to zero.
newZeroKeymapKey :: MonadIO m => m KeymapKey
newZeroKeymapKey = liftIO $ callocBytes 12 >>= wrapPtr KeymapKey

noKeymapKey :: Maybe KeymapKey
noKeymapKey = Nothing

keymapKeyReadKeycode :: KeymapKey -> IO Word32
keymapKeyReadKeycode s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Word32
    return val

keymapKeyReadGroup :: KeymapKey -> IO Int32
keymapKeyReadGroup s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 4) :: IO Int32
    return val

keymapKeyReadLevel :: KeymapKey -> IO Int32
keymapKeyReadLevel s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO Int32
    return val

type family ResolveKeymapKeyMethod (t :: Symbol) (o :: *) :: * where
    ResolveKeymapKeyMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveKeymapKeyMethod t KeymapKey, MethodInfo info KeymapKey p) => IsLabelProxy t (KeymapKey -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveKeymapKeyMethod t KeymapKey, MethodInfo info KeymapKey p) => IsLabel t (KeymapKey -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


